@@ -24,26 +24,34 @@
 #'
 #' @importFrom stats quantile
 #' @export
-rowIQRs <- function(x, rows = NULL, cols = NULL, na.rm = FALSE, ...) {
+rowIQRs <- function(x, rows = NULL, cols = NULL, na.rm = FALSE, ..., useNames = NA) {
   Q <- rowQuantiles(x, rows = rows, cols = cols,
-                    probs = c(0.25, 0.75), na.rm = na.rm, drop = FALSE, ...)
+                    probs = c(0.25, 0.75), na.rm = na.rm, useNames = useNames, drop = FALSE, ...)
+  colnames(Q) <- NULL # Not needed anymore
   ans <- Q[, 2L, drop = TRUE] - Q[, 1L, drop = TRUE]
 
   # Remove attributes
-  attributes(ans) <- NULL
+  if (is.na(useNames)) {
+    attributes(ans) <- NULL
+  }
+  
   ans
 }
 
 
 #' @rdname rowIQRs
 #' @export
-colIQRs <- function(x, rows = NULL, cols = NULL, na.rm = FALSE, ...) {
+colIQRs <- function(x, rows = NULL, cols = NULL, na.rm = FALSE, ..., useNames = NA) {
   Q <- colQuantiles(x, rows = rows, cols = cols,
-                    probs = c(0.25, 0.75), na.rm = na.rm, drop = FALSE, ...)
+                    probs = c(0.25, 0.75), na.rm = na.rm, useNames = useNames, drop = FALSE, ...)
+  colnames(Q) <- NULL # Not needed anymore
   ans <- Q[, 2L, drop = TRUE] - Q[, 1L, drop = TRUE]
 
   # Remove attributes
-  attributes(ans) <- NULL
+  if (is.na(useNames)) {
+    attributes(ans) <- NULL
+  }
+  
   ans
 }
 
