@@ -26,18 +26,41 @@
 #' @keywords array iteration robust univar
 #' @export
 rowVars <- function(x, rows = NULL, cols = NULL, na.rm = FALSE, center = NULL,
-                    dim. = dim(x), ...) {
+                    dim. = dim(x), ..., useNames = NA) {
   dim. <- as.integer(dim.)
 
   if (is.null(center)) {
     na.rm <- as.logical(na.rm)
     has_nas <- TRUE
     sigma2 <- .Call(C_rowVars, x, dim., rows, cols, na.rm, has_nas, TRUE)
+    
+    # Update names attribute?
+    if (!is.na(useNames)) {
+      if (useNames) {
+        names <- rownames(x)
+        if (!is.null(names)) {
+          if (!is.null(rows)) {
+            names <- names[rows]
+            # Zero-length attribute? Keep behavior same as base R function
+            if (length(names) == 0L) names <- NULL
+          }
+          names(sigma2) <- names
+        }
+      } else {
+        names(sigma2) <- NULL
+      }      
+    }
+    
     return(sigma2)
   }
 
   ## https://github.com/HenrikBengtsson/matrixStats/issues/187
   centerOnUse("rowVars")
+  
+  ## BACKWARD COMPATIBILITY: matrixStats (<= 0.57.0) returns names
+  ## when !is.null(center), which is tested by DelayedMatrixStats
+  ## and sparseMatrixStats
+  names <- rownames(x)
 
   # Apply new dimensions
   if (!identical(dim(x), dim.)) dim(x) <- dim.
@@ -64,7 +87,24 @@ rowVars <- function(x, rows = NULL, cols = NULL, na.rm = FALSE, center = NULL,
 
   # Nothing to do?
   if (ncol <= 1L) {
+
     x <- rep(NA_real_, times = nrow(x))
+    
+    # Update names attribute?
+    if (!is.na(useNames)) {
+      if (useNames) {
+        if (!is.null(names)) {
+          if (!is.null(rows)) {
+            names <- names[rows]
+            # Zero-length attribute? Keep behavior same as base R function
+            if (length(names) == 0L) names <- NULL
+          }
+          names(x) <- names
+        }
+      } else {
+        names(x) <- NULL
+      }      
+    }
     return(x)
   }
 
@@ -88,11 +128,6 @@ rowVars <- function(x, rows = NULL, cols = NULL, na.rm = FALSE, center = NULL,
     n <- ncol
   }
 
-  ## BACKWARD COMPATIBILITY: matrixStats (<= 0.57.0) returns names
-  ## when !is.null(center), which is tested by DelayedMatrixStats
-  ## and sparseMatrixStats
-  names <- rownames(x)
-
   validate <- validateVarsCenterFormula()
   if (!validate) {
     ## The primary formula for estimating the sample variance
@@ -102,7 +137,19 @@ rowVars <- function(x, rows = NULL, cols = NULL, na.rm = FALSE, center = NULL,
     x[is.infinite(center)] <- NaN
     x <- rowMeans(x, na.rm = na.rm)
     x <- x * (n / (n - 1))
-    names(x) <- names
+    # Preserve names attribute?
+    if (is.na(useNames) || useNames) {
+      if (!is.null(names)) {
+        if (!is.null(rows)) {
+          names <- names[rows]
+          # Zero-length attribute? Keep behavior same as base R function
+          if (length(names) == 0L) names <- NULL
+        }
+        names(x) <- names
+      }
+    } else {
+      names(x) <- NULL
+    }
     return(x)
   }
 
@@ -128,7 +175,21 @@ rowVars <- function(x, rows = NULL, cols = NULL, na.rm = FALSE, center = NULL,
   }
   
   x <- x * (n / (n - 1))
-  names(x) <- names
+  
+  # Preserve names attribute?
+  if (is.na(useNames) || useNames) {
+    if (!is.null(names)) {
+      if (!is.null(rows)) {
+        names <- names[rows]
+        # Zero-length attribute? Keep behavior same as base R function
+        if (length(names) == 0L) names <- NULL
+      }
+      names(x) <- names
+    }
+  } else {
+    names(x) <- NULL
+  }
+
   x  
 }
 
@@ -136,7 +197,7 @@ rowVars <- function(x, rows = NULL, cols = NULL, na.rm = FALSE, center = NULL,
 #' @rdname rowVars
 #' @export
 colVars <- function(x, rows = NULL, cols = NULL, na.rm = FALSE, center = NULL,
-                    dim. = dim(x), ...) {
+                    dim. = dim(x), ..., useNames = NA) {
   dim. <- as.integer(dim.)
 
   if (is.null(center)) {
@@ -144,11 +205,34 @@ colVars <- function(x, rows = NULL, cols = NULL, na.rm = FALSE, center = NULL,
     na.rm <- as.logical(na.rm)
     has_nas <- TRUE
     sigma2 <- .Call(C_rowVars, x, dim., rows, cols, na.rm, has_nas, FALSE)
+    
+    # Update names attribute?
+    if (!is.na(useNames)) {
+      if (useNames) {
+        names <- colnames(x)
+        if (!is.null(names)) {
+          if (!is.null(cols)) {
+            names <- names[cols]
+            # Zero-length attribute? Keep behavior same as base R function
+            if (length(names) == 0L) names <- NULL         
+          }
+          names(sigma2) <- names
+        }
+      } else {
+        names(sigma2) <- NULL
+      }      
+    }
+
     return(sigma2)
   }
 
   ## https://github.com/HenrikBengtsson/matrixStats/issues/187
   centerOnUse("colVars")
+  
+  ## BACKWARD COMPATIBILITY: matrixStats (<= 0.57.0) returns names
+  ## when !is.null(center), which is tested by DelayedMatrixStats
+  ## and sparseMatrixStats
+  names <- colnames(x)
 
   # Apply new dimensions
   if (!identical(dim(x), dim.)) dim(x) <- dim.
@@ -175,7 +259,24 @@ colVars <- function(x, rows = NULL, cols = NULL, na.rm = FALSE, center = NULL,
 
   # Nothing to do?
   if (nrow <= 1L) {
+    
     x <- rep(NA_real_, times = ncol(x))
+    
+    # Update names attribute?
+    if (!is.na(useNames)) {
+      if (useNames) {
+        if (!is.null(names)) {
+          if (!is.null(cols)) {
+            names <- names[cols]
+            # Zero-length attribute? Keep behavior same as base R function
+            if (length(names) == 0L) names <- NULL         
+          }
+          names(x) <- names
+        }
+      } else {
+        names(x) <- NULL
+      }      
+    }
     return(x)
   }
 
@@ -198,11 +299,6 @@ colVars <- function(x, rows = NULL, cols = NULL, na.rm = FALSE, center = NULL,
     # Assuming no missing values
     n <- nrow
   }
-
-  ## BACKWARD COMPATIBILITY: matrixStats (<= 0.57.0) returns names
-  ## when !is.null(center), which is tested by DelayedMatrixStats
-  ## and sparseMatrixStats
-  names <- names(x)
   
   validate <- validateVarsCenterFormula()
   if (!validate) {
@@ -215,7 +311,19 @@ colVars <- function(x, rows = NULL, cols = NULL, na.rm = FALSE, center = NULL,
     ## just like for stats::var() - not Inf, e.g. var(c(0,Inf)) == NaN
     x[is.infinite(center)] <- NaN
     x <- x * (n / (n - 1))
-    names(x) <- names
+    # Preserve names attribute?
+    if (is.na(useNames) || useNames) {
+      if (!is.null(names)) {
+        if (!is.null(cols)) {
+          names <- names[cols]
+          # Zero-length attribute? Keep behavior same as base R function
+          if (length(names) == 0L) names <- NULL         
+        }
+        names(x) <- names
+      }
+    } else {
+      names(x) <- NULL
+    }
     return(x)
   }
 
@@ -243,6 +351,20 @@ colVars <- function(x, rows = NULL, cols = NULL, na.rm = FALSE, center = NULL,
   }
   
   x <- x * (n / (n - 1))
-  names(x) <- names
+  
+  # Preserve names attribute?
+  if (is.na(useNames) || useNames) {
+    if (!is.null(names)) {
+      if (!is.null(cols)) {
+        names <- names[cols]
+        # Zero-length attribute? Keep behavior same as base R function
+        if (length(names) == 0L) names <- NULL         
+      }
+      names(x) <- names
+    }
+  } else {
+    names(x) <- NULL
+  }
+
   x  
 }
